/*----------------------------------------------------------------------------*\
|* program : users.c                                start date   : 03/01/1989 *|
|*                                                  last modified: 04/12/1989 *|
|* author  : Richard Hendricks                                                *|
|*         : 22 Maplewood Road, Tewksbury, MA 01876                           *|
|*         : Home: (508) 851-8447  Work: (617) 594-9692                       *|
|* compiler: Turbo C, Version 2.0, small model                                *|
|* notes   : original program name: userlist.c and original author, unknown   *|
|* function: Lists the Users Logged into the Novell Network                   *|
|* usage   : users [<search_string>]                                          *|
|* clean-up: - eliminated an unused goto reference, check:                    *|
|*         : - elininated unneeded variables, sys[8], nodes, etc              *|
|*         : - renamed function, syslog() to netcall()                        *|
|*         : - added comments to code, added white space, etc                 *|
|*         : - included some missing header file, string.h                    *|
|*         : - elininated unneeded header file, time.h                        *|
|*         : - added function prototyping for syslog()/netcall()              *|
|* imprvmts: - added substring search feature/function                        *|
|*         : - added user_knt to report                                       *|
|*         : - reworked report format                                         *|
|*         : - added register variable designations, increase speed           *|
|*         : - eliminated the default environment functions, _setenvp() -     *|
|*         :    reduce EXE size                                               *|
|*         : - added net_error() function                                     *|
\*----------------------------------------------------------------------------*/
#include <stdio.h>
#include <dos.h>
#include <string.h>
#include <ctype.h>

int  netcall( char*, char* );
void net_error( char, int );
char *stristr( char*, char* );               /* substring search, ignore case */

#ifdef __TURBOC__
_setenvp(){}              /* eliminate environment function */
#endif

char     request[ 80 ];   /* request buffer, required by Int 21h/ah=0xe3 interrupt */
char     reply[ 160 ];    /* reply buffer, contains interrupt return information   */
char     server[ 48 ];    /* name of the network server */
char     username[ 20 ];  /* user name */
char     times[ 6 ];      /* logon date and time */
int      consup;

main( argc, argv )
int      argc;
char     *argv[];
{
register int i;
register int user_knt;
int      err;

	/******************************************/
	/*                                        */
	/*    Get File Server Information         */
	/*                                        */
	/******************************************/

request[ 0 ] = '\002';  
request[ 1 ] = '\000';
request[ 2 ] = '\021';   /* subfunction 11h:  get file server information */
reply[ 0 ]   = '\202';

if( ( err = netcall( request, reply ) ) != 0 )
  {
  net_error( request[ 3 ], err );
  }

strcat( server, &reply[ 2 ] );
consup = reply[ 52 ] * 256 + reply[ 53 ];
if( strlen( server ) == 0 )
  {
  strcpy( server, "<Unknown>" );
  }
printf( "  * * * Novell Network Information * * *\n\n" );
printf( "User Information For Server %s\n\n", server );
printf( "Connection  User Name       Login Time\n" );
printf( "----------  --------------  ------------------\n" );

request[ 0 ] = '\002';
request[ 1 ] = '\000';
reply[ 1 ]   = '\000';

/******************************************/
/*                                        */
/*        Get connection information      */
/*                                        */
/******************************************/

user_knt = 0;
for( i = 1; i < consup + 1; i++ )
  {
  request[ 2 ] = '\026';   /* subfunction 16h: get connection information */
  request[ 3 ] = i;
  reply[ 0 ]   = '\076';

  if( ( err = netcall( request, reply ) ) != 0 )
    {
    net_error( request[ 3 ], err );
    }

   if( reply[ 8 ] )        /* was a username found */
     {
     username[ 0 ] = '\0';
     strcat( username, &reply[ 8 ] );
     times[ 0 ] = reply[ 56 ];
     times[ 1 ] = reply[ 57 ];
     times[ 2 ] = reply[ 58 ];
     times[ 3 ] = reply[ 59 ];
     times[ 4 ] = reply[ 60 ];
     times[ 5 ] = reply[ 61 ];
     
           /******************************************/
           /*                                        */
           /*         Get host information           */
           /*                                        */
           /******************************************/
		
     request[ 2 ] = '\023';      /* subfunction 13h: get internet address */
     reply[ 0 ]   = '\014';

     if( ( err = netcall( request, reply ) ) != 0 )
       {
       net_error( request[ 3 ], err );
       }
     user_knt++;
     if( argc > 1 )   /* user entered a search string */
       {
       if( !stristr( username, argv[ 1 ] ) )
         {
         continue;
         }
       }
     printf("    %2d      %-14s  ", i, username );
     printf("%02d/%02d/%02d  ", times[ 1 ], times[ 2 ], times[ 0 ]);
     printf("%02d:%02d:%02d\n", times[ 3 ], times[ 4 ], times[ 5 ]);
     }
  }
printf( "\n\tTotal Number of Users %4d\n", user_knt );
} /* main() */

void net_error( req, err )
char req;
int  err;
{
printf("\n\n\tNetwork Error!!! request = %d  err = %d\n", req, err );
exit( 1 );
} /* net_error() */

int netcall( req, rep )
char *req;
char *rep;
{
union REGS inregs;
union REGS outregs;

inregs.x.si = (unsigned int) req;
inregs.x.di = (unsigned int) rep;
inregs.h.ah = 0xe3;

intdos( &inregs, &outregs );

return( (int) outregs.h.al );
} /* netcall() */

char *stristr( s, s1 )
/* search in a string for another string, ignore case */
/* author: Richard Hendricks, 03/89                   */
char *s, *s1;                /* string to be searched, substring */
{
char *ss, *ss1;              /* storage for starting positions */
ss  = s;                     /* store starting positions       */
ss1 = s1;
while( *s )
  {
  if( tolower(*s) == tolower(*s1) )            /* search for a matching character */
    {
    ss = s;
    s++;
    s1++;
    while( *s && *s1 )  /* compare string and substring */
     {
     if( tolower(*s) == tolower(*s1) )
       {
       s++;
       s1++;
       }
       else
       {
       break;
       }
     }
    if( *s1 )   /* string contains substring */
      {
      return( (char *) ss );
      }
      else              /* string does not contain substring */
      {
      s = ss;           /* restore pointer starting positions           */
      s1 = ss1;         /* s1 always restored to beginning of substring */
      }                 /* s is restored to position of last match      */
    }
  s++;
  }
return( (char *) NULL );
} /* stristr() */
/* end-of-file                      users.c                       end-of-file */
